package cz.muni.pa165.client;

import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.web.servlet.context.ServletWebServerInitializedEvent;
import org.springframework.context.annotation.Bean;
import org.springframework.context.event.EventListener;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.core.Authentication;
import org.springframework.security.oauth2.client.authentication.OAuth2AuthenticationToken;
import org.springframework.security.oauth2.client.oidc.web.logout.OidcClientInitiatedLogoutSuccessHandler;
import org.springframework.security.oauth2.client.registration.ClientRegistrationRepository;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.SavedRequestAwareAuthenticationSuccessHandler;
import org.springframework.security.web.csrf.CookieCsrfTokenRepository;
import org.springframework.security.web.csrf.CsrfTokenRequestAttributeHandler;

@SpringBootApplication
@Slf4j
public class ClientApplication {

    private final ClientRegistrationRepository clientRegistrationRepository;

    @Autowired
    ClientApplication(
        ClientRegistrationRepository clientRegistrationRepository
    ) {
        this.clientRegistrationRepository = clientRegistrationRepository;
    }

    public static void main(String[] args) {
        SpringApplication.run(ClientApplication.class, args);
    }

    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity httpSecurity)
        throws Exception {
        httpSecurity
            .authorizeHttpRequests(x ->
                x
                    .requestMatchers("/", "/error")
                    .permitAll()
                    .anyRequest()
                    .authenticated()
            )
            .oauth2Login(x -> x.successHandler(authenticationSuccessHandler()))
            .logout(x ->
                x
                    .logoutSuccessUrl("/")
                    .logoutSuccessHandler(oidcLogoutSuccessHandler())
            )
            .csrf(c ->
                c
                    .csrfTokenRepository(
                        CookieCsrfTokenRepository.withHttpOnlyFalse()
                    )
                    .csrfTokenRequestHandler(
                        new CsrfTokenRequestAttributeHandler()
                    )
            );
        return httpSecurity.build();
    }

    @Bean
    public AuthenticationSuccessHandler authenticationSuccessHandler() {
        return new SavedRequestAwareAuthenticationSuccessHandler() {
            @Override
            public void onAuthenticationSuccess(
                HttpServletRequest req,
                HttpServletResponse res,
                Authentication auth
            ) throws ServletException, IOException {
                if (
                    auth instanceof OAuth2AuthenticationToken token &&
                    token.getPrincipal() instanceof OidcUser user
                ) {
                    log.debug(
                        "********************************************************"
                    );
                    log.debug(
                        "* user successfully logged in                          *"
                    );
                    log.debug(
                        "********************************************************"
                    );
                    log.info("user.issuer: {}", user.getIssuer());
                    log.info("user.subject: {}", user.getSubject());
                    log.info("user.fullName: {}", user.getFullName());
                    log.info("user.givenName: {}", user.getGivenName());
                    log.info("user.familyName: {}", user.getFamilyName());
                    log.info("user.gender: {}", user.getGender());
                    log.info("user.email: {}", user.getEmail());
                    log.info("user.locale: {}", user.getLocale());
                    log.info("user.zoneInfo: {}", user.getZoneInfo());
                    log.info(
                        "user.preferredUsername: {}",
                        user.getPreferredUsername()
                    );
                    log.info("user.issuedAt: {}", user.getIssuedAt());
                    log.info(
                        "user.authenticatedAt: {}",
                        user.getAuthenticatedAt()
                    );
                    log.info(
                        "user.claimAsListString(\"eduperson_scoped_affiliation\"): {}",
                        user.getClaimAsStringList(
                            "eduperson_scoped_affiliation"
                        )
                    );
                    log.info(
                        "user.attributes.acr: {}",
                        user.<String>getAttribute("acr")
                    );
                    log.info("user.attributes: {}", user.getAttributes());
                    log.info("user.authorities: {}", user.getAuthorities());
                }
                super.onAuthenticationSuccess(req, res, auth);
            }
        };
    }

    private OidcClientInitiatedLogoutSuccessHandler oidcLogoutSuccessHandler() {
        OidcClientInitiatedLogoutSuccessHandler successHandler =
            new OidcClientInitiatedLogoutSuccessHandler(
                clientRegistrationRepository
            );
        successHandler.setPostLogoutRedirectUri("http://localhost:8080/");
        return successHandler;
    }

    @EventListener
    public void onApplicationEvent(
        final ServletWebServerInitializedEvent event
    ) {
        log.info("**************************");
        log.info("visit http://localhost:{}/", event.getWebServer().getPort());
        log.info("**************************");
    }
}
